/**
 * (c) Universidad de Oviedo.
 * 
 * This is part of the "SQLTools" set of tools for database testing.
 * See SQLTools terms of use at: http://in2test.lsi.uniovi.es/sqltools/common/license.aspx
 * 
 * Sample program for generating and executing FPC coverage rules
 * from a program whithout the use of the SQLRules user interface
 * 
 * @author Javier Tuya
 * @date December 2008
 * 
 * THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. 
 * EXCEPT WHEN OTHERWISE STATED IN WRITING, THE COPYRIGHT HOLDER PROVIDE THE PROGRAM "AS IS" 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
 * QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, 
 * YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
 * 
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING, ANY COPYRIGHT 
 * HOLDER WILL BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL 
 * OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM 
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES 
 * SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER 
 * PROGRAMS), EVEN IF SUCH HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 */
package in2test.samples.sqlrules.java;
import in2test.sql.query.*;
import in2test.application.common.*;
import in2test.application.services.*;
import java.awt.event.*;
import javax.swing.*;


public class RunSQLRulesSample extends JPanel implements ActionListener {
	//Customize here the data access parameters required by the JDBC driver
	//Below configuration assumes the Oracle sample database HR
	//sample query: 
	//select e.*,d.department_name from employees e left join departments d on e.department_id=d.department_id where employee_id=100
	private final String DRIVER="oracle.jdbc.driver.OracleDriver";
	private final String URL="jdbc:oracle:oci:@localhost:1521:ORCL";
	private final String USER="HR";
	private final String PASSWORD="";
	//Customize here the parameters required by the java.sql.DatabaseMetadata methods
	private final String CATALOG="";
	private final String SCHEMA="HR";

	//User interface objects and methods
	static final long serialVersionUID=0;
	public  JFrame mainFrame;
	private JLabel lblSql=new JLabel("Query:     ");
	private JLabel lblSchema=new JLabel("Schema: ");
	private JLabel lblRules=new JLabel("Rules (XML): ");
	private JLabel status=new JLabel("");
	private JTextArea sql=new JTextArea(5,60);
	private JTextArea schema=new JTextArea(7,60);
	private JTextArea rules=new JTextArea(12,60);
	private JButton getSchema=new JButton("Get Schema");
	private JButton generate=new JButton("Get Rules");
	private JButton runRules=new JButton("Run Rules");

	public RunSQLRulesSample(String nombre) {
		mainFrame = new JFrame(nombre);
		//layout
		this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));

		Box bxSql=Box.createHorizontalBox();
		bxSql.add(lblSql);
		JScrollPane spSql=new JScrollPane(sql);
		bxSql.add(spSql);
		this.add(bxSql);

		Box bxGenerate=Box.createHorizontalBox();
		bxGenerate.add(getSchema);
		bxGenerate.add(generate);
		bxGenerate.add(runRules);
		bxGenerate.add(status);
		this.add(bxGenerate);

		Box bxSchema=Box.createHorizontalBox();
		bxSchema.add(lblSchema);
		JScrollPane spSchema=new JScrollPane(schema);
		bxSchema.add(spSchema);
		this.add(bxSchema);

		Box bxRules=Box.createHorizontalBox();
		bxRules.add(lblRules);
		JScrollPane spRules=new JScrollPane(rules);
		bxRules.add(spRules);
		this.add(bxRules);

		//install event listener
		getSchema.setName("getSchema");
		getSchema.setActionCommand("getSchema");
		getSchema.addActionListener(this);
		generate.setName("generate");
		generate.setActionCommand("generate");
		generate.addActionListener(this);
		runRules.setName("runRules");
		runRules.setActionCommand("runRules");
		runRules.addActionListener(this);
		// pack and show
		mainFrame.setContentPane(this);
		mainFrame.pack();
		mainFrame.show();//(true);
	}

	/**
	 * Process button events
	 */
	public void actionPerformed(ActionEvent e) {
		try {
			System.out.println("Executing command: " + e.getActionCommand());
			status.setText("Executing command: " + e.getActionCommand());
			mainFrame.update(mainFrame.getGraphics());
			SQLToolsConfig.configure(); //this is required to use the web service Facade
			if ("getSchema".equals(e.getActionCommand())) {
				//Creates a facade to get the xml string that includes the database schema
				SQLQueryFacade facade=new SQLQueryFacade(DRIVER,URL,USER,PASSWORD);
				String sSchema=facade.getSchemaXml(CATALOG, SCHEMA, this.sql.getText());
				this.schema.setText(sSchema);
				this.schema.select(0,0);
			} else if ("generate".equals(e.getActionCommand())) {
				//Creates a facade for the web service that obtains the rules
				this.rules.setText("");
				String sRules="";
				SQLFpcWSFacade wsFpc=new SQLFpcWSFacade();
				sRules=wsFpc.getRules(this.sql.getText(), this.schema.getText(), ""); //"<options><formatsql/></options>");
				//this is for generating mutants instead of FPC rules
				//SQLMutationWSFacade wsMutation=new SQLMutationWSFacade();
				//sRules=wsMutation.getMutants(this.sql.getText(), this.schema.getText(), "",true);
				this.rules.setText(sRules);
				this.rules.select(0,0);
			} else if ("runRules".equals(e.getActionCommand())) {
				//Creates a facade to process the rules and get the results
				SQLQueryFacade facade=new SQLQueryFacade(DRIVER,URL,USER,PASSWORD);
				String sRules=facade.runRules(this.rules.getText(),"",30); //empty parammeter may be filled with query parameters, query timeout set to 30
				this.rules.setText(sRules);
				this.rules.select(0,0);
			}
			System.out.println("Done");
			status.setText("Done");
		} catch (Exception ex) {
			System.out.println("Exception: " + ex.getMessage());
			ex.printStackTrace();
			status.setText("Exception: " + ex.getMessage());
			msgBox("Exception: " + ex.toString());
		}
	}
	protected void msgBox(String msg) {
		JOptionPane.showMessageDialog(mainFrame,msg,
				"Exception Message",JOptionPane.ERROR_MESSAGE);
	}
	public static void main(String[] args) {
		new RunSQLRulesSample("Example");
	}
}
